<?php

/**
 * This class is a sample how to work with aMember Am_Record and Am_Table
 * classes
 * It is better to use Am_Grid instead, but we are keeping it simple in this example
 */
class Example_DaoController extends Am_Mvc_Controller
{
    /** @var ExampleTable */
    protected $table;

    /**
     * This method is executed before any action
     * you can do common actions here.
     * if you throw an exception from this point, other methods will not be executed
     */
    public function preDispatch()
    {
        parent::preDispatch();
        // once you access Am_Di->somethingTable it will automatically look
        // corresponding table class and creates singleton for it
        $this->table = $this->getDi()->exampleTable;
    }

    function indexAction()
    {
        // now we will fetch all records from the table
        $records = $this->table->findBy([]); // just select all records without filtering


        echo "<table border=1>";
        echo "<tr><th>Id</th><th>Title</th><th>Comment</th><th>&nbsp;</th></tr>\n";
        $root = $this->getDi()->url("example/dao");
        foreach ($records as $example)
        {
            /* @var $example Example */
            echo "<tr>";
            echo "<td>" . $example->example_id . "</td>";
            echo "<td>" . Am_Html::escape($example->title) . "</td>";
            echo "<td>" . Am_Html::escape($example->comment) . "</td>";
            echo "<td><a href='$root/delete?id={$example->example_id}'>Delete</a></td>";
            echo "</tr>\n";
        }
        echo "</table>";
        echo "<a href='$root/insert'>Insert new record</a>";
    }

    function insertAction()
    {
        // how to insert new record:
        /* @var $example Example */
        $example = $this->table->createRecord();
        $example->title = md5(rand());
        $example->comment = sha1(rand());
        $example->insert(); // or call $example->save();
        // how to edit record
        $example->title = $example->title . '-XX';
        $example->update(); // or call $example->save();
        $this->_redirect('example/dao');
    }

    function deleteAction()
    {
        $id = $this->_request->getInt('id');
        // or you may use
        // $this->_request->getFiltered('id'); for alphanumeric strings
        // $this->_request->get('id'); to get raw unfiltered input
        if (!$id)
            throw new Am_Exception_InputError("Empty Id");
        // other types of exceptions logs an error, Am_Exception_InputError does not
        $example = $this->table->load($id); // will throw an exception if not found
        $example->delete();
        $this->_redirect('example/dao');
    }

    function sample()
    {
        // there are other sample usages:
        // returns array of Example objects
        $records = $this->table->findBy(['title' => 'XX']); // AND
        // returns single Example object or null
        $record = $this->table->findFirstBy(['example_id' => '11', 'title' => 'XX']); // AND
        //
        $this->table->deleteBy(['comment' => 'XX']);
        // returns array of Example objects
        $records = $this->table->selectObjects("SELECT * FROM ?_example WHERE example_id>2");

        // single-record operations
        $example = $this->table->load(1, false); // false to do not throw exception if not found
        if (!$example) return;
        $example->title = 'XX'; // change title and save
        $example->save();
        $example->pk(); // get primary key - example_id for this object
        $example->updateQuick('title', 'YY'); // run UPDATE am_example title='YY' WHERE example_id=1
        $example->delete(); // delete the object
        // create new object
        $example = $this->table->createRecord();
        $example->title = 'xx';
        $example->comment = 'aa';
        $example->insert();
    }
}